
#ifndef KDELIBS4SUPPORT_EXPORT_WRAPPER_H
#define KDELIBS4SUPPORT_EXPORT_WRAPPER_H

#include "kdelibs4support_export_internal.h"

#ifndef KDELIBS4SUPPORT_NO_DEPRECATED_NOISE
#define KDELIBS4SUPPORT_DEPRECATED_NOISE KDELIBS4SUPPORT_DEPRECATED
#define KDELIBS4SUPPORT_DEPRECATED_EXPORT_NOISE KDELIBS4SUPPORT_DEPRECATED_EXPORT
#else
#define KDELIBS4SUPPORT_DEPRECATED_NOISE
#define KDELIBS4SUPPORT_DEPRECATED_EXPORT_NOISE KDELIBS4SUPPORT_EXPORT
#endif

#endif
